//Pseudo C code for Intellabeam controllers


//----------------------------------------------------------------------------
void Interrupt(void)
//Branch to interrupt handlers
	//BRK_PIN
	
	//Timer10mS
	{
		Timer10ms++;
	}
//----------------------------------------------------------------------------
void main(void)
{
	//Setup baudrate
	//Setup pins
	//Init vars

}
//----------------------------------------------------------------------------
void Brk_Interrupt(void)
{
	timeout=10;
	//Ensure BRK pin stays low for 10 cycles
	while(timeout && !PIN_BRK)
	{
		timeout--;
		wait_us(1);
	}
	if (timeout) return; 		//Quit if glitched
	
	timeout=10;
	//Wait for BRK pin to go high, or timeout
	while(timeout && !PIN_BRK)
	{
		timeout--;
		wait_us(10);
	}	
	if(!timeout) return;			//Quit if timeout
	
	//Wait for RX flag to show data received, or timeout
	PIR1bits.RCIF=0;		//Clear flag
	timeout=255;
	while(timeout & !PIR1bits.RCIF)
	{
		timeout--;
		wait_us(10);		
	}
	if(!timeout) return;			//Quit if timeout
	
	//Check first byte = 0x00
	PIR1bits.RCIF=0;		//Clear flag
	if(RCREG!=0x00) return;		//Quit if wrong value
	
	Channel_Counter=DMX_CHANNEL;
	if(Channel_Counter>0)
	{
		timeout=100;
		while(Channel_Counter)
		{
			//Wait for next byte
			PIR1bits.RCIF=0;		//Clear flag
			while(timeout & !PIR1bits.RCIF)
			{
				timeout--;
				wait_us(10);		
			}
			if(!timeout) return;			//Quit if timeout
			Channel_Counter--;
		}
	}
	
	//Now ready to receive our 8 bytes of control data
	
	Byte_Counter=0;
	timeout=100;
	while(Byte_Counter<8)
	{
		//Wait for next byte
		PIR1bits.RCIF=0;		//Clear flag
		while(timeout & !PIR1bits.RCIF)
		{
			timeout--;
			wait_us(10);		
		}
		if(!timeout) return;			//Quit if timeout
		DMX_Buffer[Byte_Counter]=RCREG;
		Byte_Counter++;
	}
	
	//Finished gathering data
}
//----------------------------------------------------------------------------
void Update_Motors(void)
{
	//If something has changed, recalculate motor values
	i=0;
	while(i<8)
	{
		if(DMX_Buffer[i]!=Last_DMX_Buffer[i]) 
		{
			Calculate_Motor_Values();
			Auto_Update_Counter=0;
			i=99;	//Force exit
		}
		i++;
	}
	if(i=8)
	{
		//Passed without updating - nothing changed.
		//If nothing has changed 'n' times in a row, then update anyway
		Auto_Update_Counter++;
		if(Auto_Update_Counter>=0x30) 
		{
			Calculate_Motor_Values();
			Auto_Update_Counter=0;
		}
	}
	
	return;
}
//----------------------------------------------------------------------------
void Calculate_Motor_Values(void)
{
	//Arrange bytes in correct order and prefix with 0x20
	//For pan and tilt, 0x1234 converts to '0x22, 0x21, 0x24, 0x23'
	long ltemp;
	ltemp=((ValuePan & 0x0f00)>>16) + 0x20;
	Motor_Buffer[0]=(unsigned char) ltemp;
	ltemp=((ValuePan & 0xf000)>>24) + 0x20;
	Motor_Buffer[1]=(unsigned char) ltemp;
	ltemp=((ValuePan & 0x000f)) + 0x20;
	Motor_Buffer[2]=(unsigned char) ltemp;
	ltemp=((ValuePan & 0x00f0)>>8) + 0x20;
	Motor_Buffer[3]=(unsigned char) ltemp;
	
	ltemp=((ValueTilt & 0x0f00)>>16) + 0x20;
	Motor_Buffer[4]=(unsigned char) ltemp;
	ltemp=((ValueTilt & 0xf000)>>24) + 0x20;
	Motor_Buffer[5]=(unsigned char) ltemp;
	ltemp=((ValueTilt & 0x000f)) + 0x20;
	Motor_Buffer[6]=(unsigned char) ltemp;
	ltemp=((ValueTilt & 0x00f0)>>8) + 0x20;
	Motor_Buffer[7]=(unsigned char) ltemp;
	
	Motor_Buffer[8]=0x83;
	Motor_Buffer[9]=ValueGobo+0x20;
	Motor_Buffer[10]=ValueColour+0x20;
	Motor_Buffer[11]=ValueIris+0x20;
	Motor_Buffer[12]=ValueShutter+0x20;
	if(Timer10ms>=50)
	{
		Motor_Buffer[13]=0x21;	
	}else{
		Motor_Buffer[13]=0x20;
	}
	//Send bytes
}
//----------------------------------------------------------------------------
const rom unsigned char TBL_07C7 = {	081h,071h,061h,051h,041h,031h,021h,011h,
					010h,020h,030h,040h,050h,060h,070h,080h,
					00bh,00ah,009h,008h,007h,006h,005h,004h,
					003h,002h,001h,000h,000h,000h,000h,000h};
const rom unsigned char TBL_07E7 = {	080h,070h,060h,050h,040h,030h,020h,010h,
					011h,021h,031h,041h,051h,061h,071h,081h,
					00bh,00ah,009h,008h,007h,006h,005h,004h,
					003h,002h,001h,000h,000h,000h,000h,000h};	
void Convert_DMX_Values(void)           
{                                       
	for(i=0; i<8; i++)              
	{                               
		Last_DMX_Buffer[i]=DMX_Buffer[i];
	}                               
	//Pan                           
	ValuePan = DMX_Buffer[0]*0x0c;  
	//Tilt                          
	ValueTilt = DMX_Buffer[1]*0x04; 
	                                
	//Colour                        
	iptr = (DMX_Buffer[2]*0x1c)>>8; 
	iptr &= 0x001f;                 
	ValueColour=TBL_07C7[iptr];     
	                                
	//Gobo                          
	iptr = (DMX_Buffer[3]*0x1c)>>8; 
	iptr &= 0x1f;                   
	ValueGobo=TBL_07E7[iptr];       
	                                
	//Shutter                       
	ValueShutter = (DMX_Buffer[4]*10)>>8;
	if(ValueShutter!=0x09)          
	{                               
		//swap ValueShutter     
	}else{                          
		ValueShutter=1;	        
	}                               
	                                
	//Dimmer                        
	ValueDimmer = (DMX_Buffer[5]*100)>>8;
	if(ValueDimmer<10) ValueShutter=0;
	
	//Iris
	ValueIris=(Dmx_Buffer[6]*100)>>8;

}