;DMX RX.asm
Interrupt_Vector	EQU .8
Common_Ram_Base	 	EQU	H'70'
Ram_Base	 		EQU	H'20'

Number_Of_Channels  EQU .8
startcode			EQU	.0
; ***********	Define Flags   *************
;		IFlag
Break_Detected	equ 0	; Break detected
Start_Detected	equ 1	; Start Byte detected
Valid_Channel	equ 2	; Reached valid channels
No_DMX_Data		equ 7	; No recieved DMX Data

;		Flag

longcount	equ 0	; Long count flag (2.5s) when activated		
presetmode	equ 1	; Preset Mode on no DMX data
curve		equ 2	; S Curve selected
switch		equ 3	; Switch curve selected
DMX_Data	equ 4	; New DMX Data recieved
phase_done	equ 5	; Flag to show mains phase complete
dmx_error	equ 6	; Error detected in DMX tranmission
mains_fail	equ	7	; Maind Failure Flag

;***** VARIABLE DEFINITIONS
	CBLOCK Common_Ram_Base
	
	Int_W
	Int_Status
	Int_PCLath
	Int_FSR
	ee_address
	ee_data

	ENDC

	CBLOCK Ram_Base

;	Interupt Variables
	Iflag
	IH_Channel_Count_L
	IH_Channel_Count_H
	IH_Channels_Recieved
	IH_Temp
	IH_Long_Count		; Long duration count (2.5s)
	IH_Timer_Count
	IH_position_count
	IH_channel1cnt		; intertupt counts
	IH_channel2cnt
	IH_channel3cnt
	IH_channel4cnt

;	Programme variables
	countervalue		; Preset to load timer 1 from
	position_count		; Number of positions in last mains pulse
	Current_Preset		; Current Preset being displayed
	flag				; general flags
	flag2				; General flag
	dflag				; Display Flags
	Delay_Count			; Counter used in delay routien
	display_count		; Used for 7 Seg display
	displayvalue		; Used for 7 Seg display
	olddisplayvalue
	shadowa				; Port A shadow register
	oldporta			; Old Port a Value
	process_temp		; Process Tempory value
	tbllo				; Table Low Value
	;DMX_Start_Addr_L	; DMX Start Address
    ;DMX_Start_Addr_H
    DMX_Start_Code		; DMX Start Code
    Channels			; Number of channels 
	
	Channel_Values : (Number_Of_Channels+2)
	
	ENDC
	MYDATA UDATA
	DMX_Start_Addr_L RES 1
	DMX_Start_Addr_H RES 1
	GLOBAL DMX_Start_Addr_L
	GLOBAL DMX_Start_Addr_H
;-----------------------------------------------------

	include <p18f452.inc>
	ORG     Interrupt_Vector   	; interrupt vector location
	goto	asm_int_handler
	
	UDATA_ACS
	CODE
	
asm_int_handler

		CLRF	DMX_Start_Addr_H,0
		MOVWF   Int_W,0             	;; Save PIC state
        SWAPF   STATUS,W,0
        CLRF    STATUS,0
        MOVWF   Int_Status,0
        MOVF    PCLATH,W,0
        MOVWF   Int_PCLath,0
        CLRF    PCLATH,0
        ;BCF     STATUS,IRP
        MOVF    FSR0L,W,0
        MOVWF   Int_FSR,0

IH_Interrupt_Poll:
		;btfsc 	INTCON,T0IF		;; Test for timer0 interupt
		;goto 	IH_Timer0		;; Yes, Goto timer0 Handler
		btfsc 	PIR1,RCIF,0		;; USART Interupt Pending ?
		goto	IH_Check_USART	;; Yes, Goto Usart handler
		;btfsc 	INTCON,INTF		;; Test for RB0 Interupt
		;goto 	IH_RB0
		;btfsc	PIR1,TMR1IF		;; Timer 1 interupt ??
		;goto	IH_Timer1

IH_Exit:  
		movf    Int_FSR,W,0		;; Restore PIC state
        movwf   FSR0L,0
        movf    Int_PCLath,W,0
        movwf   PCLATH,0
        swapf   Int_Status,W,0
        movwf   STATUS,0
        swapf   Int_W,F,0
        swapf   Int_W,W,0
        retfie  0					;; Return from interrupt
         	
IH_Check_USART:
	   	bcf 	PIR1,RCIF,0			; Clear Interupt flag
		;bsf	Data_Rec_LED		; Set DMX Data LED
			
IH_Check_HW_Overrun:
	    btfsc   RCSTA,OERR,0      ; Has a HW overrun occurred ?
        goto    IH_HW_Overrun   ; Yes, do a HW reset

IH_Check_9th_bit:
		clrf	IH_Timer_Count,0		; Clear Timer count
		btfss 	RCSTA,RX9D,0			; Dose 9th Bit of data = 1, eg Break
		goto 	IH_Wait_Break		; Yes, start receiving again

IH_Check_SW_Overrun:
	    btfsc   RCSTA,FERR,0         	; Has a framing error occurred ?
        goto    IH_SW_Overrun		; Yes, do a Usart reset

IH_Test_Flags:
		btfsc 	Iflag,Valid_Channel,0	; Test to see if receiving valid channels
		goto	IH_Valid_Channel	; Yes, get next channel
		btfsc	Iflag,Start_Detected,0	; Test to see it start byte has arrived
		goto	IH_Wait_Channel		; Yes, Check Channel counts
		btfsc	Iflag,Break_Detected,0	; Test to see if break has arrived
		goto	IH_Wait_Start		; Yes, wait for start byte

IH_Wait_Break:
		clrf	Iflag,0				; Clear all interupt flags
		movf	RCREG,W	,0			; Move recieved data in to w
		btfsc	RCSTA,RX9D,0			; Test if break has arrived
		goto	IH_Interrupt_Poll	; No,  Break not detected
		btfsc	STATUS,Z,0			; Yes, Dose detected break =0 ?
		bsf 	Iflag,Break_Detected,0	; Yes, set flag to indicate break arrived
		goto	IH_Interrupt_Poll

IH_Wait_Start:
		clrf	IH_Channels_Recieved,0	; Clear number of channels recieved
		clrf	IH_Channel_Count_L,0		; Clear Low channel count
		clrf	IH_Channel_Count_H,0		; Clear High channel count
		bsf		Iflag,Start_Detected,0	; Set flag to indicate Start Byte arrived
		movf	RCREG,W	,0				; Check start code = 0
		btfss	STATUS,Z,0				;
		clrf	Iflag,0					; if No then clear flags and wait for next break
		goto	IH_Interrupt_Poll

IH_Wait_Channel:
		movf	RCREG,W	,0				; Store Recieved value in temp
		movwf	IH_Temp,0
		incf	IH_Channel_Count_L,F,0	; Increase channel count
		btfsc	STATUS,Z,0				; for both Low and High bytes
		incf	IH_Channel_Count_H,F,0	;
		movf	IH_Channel_Count_H,W,0	; Compare the high byte of channel count to 			
		xorwf	DMX_Start_Addr_H,W,0		; DMX start address
		btfss	STATUS,Z,0				; If Not equal then exit
		goto	IH_Interrupt_Poll
		movf	IH_Channel_Count_L,W,0	; Compare the low byte of channel count to 
		xorwf	DMX_Start_Addr_L,W,0		; DMX Start address
		btfss	STATUS,Z,0				; if not equal then exit
		goto	IH_Interrupt_Poll
		bsf		Iflag,Valid_Channel,0		; if address equal then set valid channels		
		movf	IH_Temp,W,0
		movwf	Channel_Values,0			; Store value in first channel
		incf	IH_Channels_Recieved,F,0	; and increase count
		goto	IH_Interrupt_Poll

IH_Valid_Channel:
		movlw	Channel_Values			; Load address of memory block
		addwf	IH_Channels_Recieved,W,0	; add channel offset
		movwf	FSR0L,0						; and move in to FSR
		movf	RCREG,W					; Store result in the 	
		movwf	INDF0					; channel position in memory
		movf	IH_Channels_Recieved,W,0	;
		incf	IH_Channels_Recieved,F,0	; Increase count (Note No effect on w !)
		xorwf	Channels,W,0				; Check if all channels recieved
		btfss	STATUS,Z,0				; 
		goto	IH_Interrupt_Poll		; No, then return to interupt poll
		clrf	Iflag,0					; Yes, Clear flags and wait for break
	;if basic == 1
	;	bsf		Data_Rec_LED			; Set DMX Data LED
	;endif
		bsf		flag,DMX_Data,0			; Set DMX Data recieve flag
		goto 	IH_Interrupt_Poll		; exit back to Handler		

IH_HW_Overrun:
    	bcf     RCSTA,CREN            	; reset UART
        bsf     RCSTA,CREN            	; receive logic
		bsf  	flag,dmx_error,0        	; Turn on hardware overrun LED
	;if basic == 1
	;	bsf		Data_Error_LED			; Turn on DMX Error led
	;endif
		goto 	IH_Check_9th_bit

IH_SW_Overrun:
	    bcf     RCSTA,CREN            	; reset UART
        bsf     RCSTA,CREN            	; receive logic
		bsf		flag,dmx_error,0			; Turn on software Overrun Led
	;if basic == 1	
	;	bsf		Data_Error_LED			; Turn on DMX Error led
	;endif
		goto    IH_Test_Flags
		
		GLOBAL asm_int_handler
	
		
		END
;	_endasm
		
